'******************************************************************************
'   by Wolfgang Hugemann, autorenteam GbR
' www.unfallrekonstruktion.eu
'
' This script mounts several input images (most probably video frames, stored as JPEGs)
' to two (or more) film strips with perforation and white space inbetween.
' The input filesnames have to be handed to the script as command line arguments,
' most effectively by either Drag & Drop or via SendTo.
' The script should typically be stored in the SendTo directory.
' The images must all have the same dimensions (as typically is the case for video frames).
'
' The ordering of the images is automatically chosen such that it will fit best on a portrait
' sheet (DIN-A4). The default ordering can be overwritten by placing a text file named 'strip.txt'
' within the directory that holds the images.
'
' The images will be ordered in columns (as indicated by the perforation) according to alphabetical order
' of the filenames. Video frames should therefore be named according to their chronological order.
'
'  Version 2009-07-14
'******************************************************************************
'
Option Explicit
' Definition of the geometry
' Nomenclature:
' f*** = fraction
' *Fr* = frame 
' *P** = perforation
' *PH* = perforation hole
'
Const fFrDY  = 0.05  ' height of the strip between the images, as percentage of the image height
Const fWx    = 0.2   ' width of the white strip between the film strips, as percentage of the image width
Const fPx    = 0.09  ' width of the perforation, as percentage of the image width
Const fPHx   = 0.5   ' width of the perforation hole, as percentage of the perforation strip width
Const fPHAR  = 0.75  ' AspectRatio of the perforation hole
Const fPHR   = 0.15  ' fillet radius of the perforation hole
Const fPHDy  = 1.4   ' distance of the perforation hole, as percentage of the hole height

Const ScalingThreshold = 2.4 ' scale factor threshold for asking for scaling to DIN A3

' filename
Const strConv    = "IMconvert"  ' IM's Convert command line tool
const strOut     = "film"       ' output file 
Const strPH      = "hole.miff"  ' temporary file for perforation hole
Const strPs      = "strip.miff" ' temporary file for tiling the perforation holes
Const strP       = "perf.miff"  ' temporary file for the perforation strip
Const strW       = "white.miff" ' temporary file for the white strip between the film strips
Const strPre     = "F-"         ' name prefix of the temporary files for the single film strips
const strExt     = ".miff"      ' filename extension for all temporary files MIFF = Magic Image File Format
const strTxtFile = "strip.txt"  ' name of the control file (searched for in the same directory as the frames)
const strComment = "'"          ' comment sign in the control file

Dim Y
Dim FName() ' Filenames (i.e. command line arguments)
Dim wsh, fs, objExec, objFile ' Objects
Dim FrY, FrDx, FrDy ' Frame
Dim Px, Py, PHx, PHy, PHOx, PHOy, PHAR, PHR, PHDY 'Perforation
Dim Wx ' white strip
Dim i, j, NArgs, NRows, NCols 'Integers
Dim Temp, PDir, cbase, command 'Strings
Dim B2H 'V Aspect ratio
Dim bCtrlFile ' Boolean value: control file present?
Dim strline, Pos
Dim strFileExt

Dim Width, Height, dpi
'
' Usable portion of the DIN-A4 protrait page
Width  = 200 ' mm
Height = 280 ' mm
dpi    = 150 ' dpi
B2H = Width / Height
Y = Height / 25.4 * dpi ' total height; cannot be defined by a constant, as it will have to be adjusted


Dim strf  ' single string from the following array
Dim ip, jp ' position of the special character within the string
Dim F1, F2 ' 1. + 2. factor 
Dim fw, fh ' single frames: width and height
Dim Factors ' possible ordering of the frames x*y according to the number of files supplied 2 ... 30
            ' alternative ordering are dived by a semicolon
Factors = array("","","","2*2;","","2*3;","","2*4;","3*3;","2*5;", _
                "","2*6;4*3;","","2*7;","3*5;","2*8;4*4;","","2*9;3*6;","","2*10;4*5;", _
					 "3*7;","2*11;","","2*12;3*8;4*6;","5*5;","2*13;","3*9;","2*14;4*7;","","2*15;5*6;")
					 
Set wsh = CreateObject("Wscript.Shell")
Set fs = CreateObject("Scripting.FileSystemObject")

NArgs = WScript.Arguments.Count
if NArgs < 2 Then 
	MsgBox "Please select at least two files.", vbExclamation
	WScript.Quit
end if
Redim FName(NArgs-1)

'  Only filenames must be supplied
For i = 0 to NArgs - 1
	FName(i) = WScript.Arguments(i)
	if not fs.FileExists(FName(i)) then
		MsgBox """" & FName(i) & """ is a directory.", vbExclamation
		WScript.Quit
	End if
next

' Filenames must be sorted as they are handed in random order by Drag & Drop or SendTo (Bubble Sort)
for i = 0 to NArgs - 1
	for j = i + 1 to NArgs - 1
		if FName(i) > FName(j) then
			Temp = FName(i)
			FName(i) = FName(j)
			FName(j) = Temp
		end if
	next
next

' determine the name of the folder which contains the frames
PDir = fs.getParentFolderName(FName(0)) & "\"
Wsh.CurrentDirectory = PDir

' determine the file type (i.e. extension)
Pos = InStrRev(FName(1),".")
strFileExt = Mid(FName(1), Pos + 1)

' check whether a control file exist and if so, read the column count from it
If fs.FileExists(strTxtFile) then
	Set objFile = fs.OpenTextFile(strTxtFile, 1)
	bCtrlFile = True
	strLine = objFile.ReadLine
	if not len(strLine) = 0 then
		Pos = InStr(strLine, strComment)
		if not Pos = 0 then
			if not Pos = 1 then
				strline = left(strline, Pos - 1)
			end if
		end if
		NCols = strline
	end if
	objFile.close
else
	bCtrlFile = False
end if

' Dateiamen in Anfhrungszeichen setzen, wegen mglicher Leerzeichen
For i = 0 to NArgs - 1
	FName(i) = """" & FName(i) & """"
next

' determine the dimensions of the single frames
command = "cmd /k identify -format %wx%h " & FName(0)
Set objExec = wsh.Exec(command)
strf = objExec.StdOut.Read(10)
ip = Instr(strf,"x")
fw = Left(strf, ip - 1)
fh =  Mid(strf, ip + 1)

if bCtrlFile then
	NRows = Fix(Nargs / NCols)
else
'  determine optimal ordering, depending on the number of image files handed over to the script
Dim Last, Prod, Tr, R, Rest
Rest = 10000
Last = False
strf = 1 & "*" & Nargs & ";" & Factors(NArgs-1)
For j = 1 To 4
   ip = InStr(strf, ";")
   If ip <> 0 Then
		Prod = Left(strf, ip - 1)
      strf = Mid(strf, ip + 1)
	else
		Exit for
	end if
   jp = InStr(Prod, "*")
   F1 = Left(Prod, jp - 1)
   F2 = Mid(Prod, jp + 1)
   Tr = (F1 * fw) / (F2 * fh)
   R = Abs(B2H - Tr)
   If R < Rest Then
		NRows = F2
      NCols = F1
      Rest = R
   End If
   Tr = (F2 * fw) / (F1 * fh)
   R = Abs(B2H - Tr)
   If R < Rest Then
      NRows = F1
      NCols = F2
      Rest = R
   End If
Next
End if

' check whether the (larger) DIN A3 sheet format should rather be used
IF NRows * fh / Y > ScalingThreshold then
	if MsgBox("Should we scale to DIN A3?", 36, "Target size") = 6 then Y = Y * 1.414
end if
		
' Height of the images
FrY = Round(Y/(NRows * (1 + 2 * fFrDY)))
' Height of the srips between the single images
FrDY = Round(0.5 * fFrDY * FrY)
' Determine total height anew
Y = NRows * (FrY + 2 * FrDY)
' Distance to perforation
FrDX = Round(0.5 * FrDY)

' Stub for the montage command 
cbase = "montage -tile 1x" & NRows & " -geometry x" & FrY & "+" & FrDX & "+" & FrDY & " -background brown "

' Generate a strip for each column
For i = 0 to NCols - 1
	command = cbase
	for j = 0 to NRows - 1
		command = command & " " & FName(i * NRows + j)
	next
	command = command & " " & strPre & i & strExt
	wsh.run command, 7, true
next

Px  = fPx * Fry * sqr(fw * fh) / fh ' width of the perforation strip
PHx = fPHx  * Px          ' width of the perforation hole
PHy = fPHAR * PHx         ' height of the perforation hole
Py  = (1 + fPHDy) * PHy   ' height of the bitmap for the perforation hole
PHR = fPHR * PHx          ' fillet radius of the perforation hole

' rounding to integer values
Px  = 2 * Round(0.5 * Px)
PHx = 2 * Round(0.5 * PHx)
PHy = 2 * Round(0.5 * PHy)
Py  = 2 * Round(0.5 * Py)
PHR = Round(PHR)
PHOx = Round(0.5 * (Px-PHx)) - 1
PHOy = Round(0.5 * (Py-PHy)) - 1

command = """roundRectangle " & PHOx & "," & PHOy & "," & (PHOx + PHx + 1) & "," & (PHOy + PHy + 1) & "," & PHR & "," & PHR & """"
command = strConv & " -size " & Px & "x" & Py & " xc:brown -fill white -draw " & command & " " & strPH
wsh.run command, 7, true

Command = strConv & " -size " & Px & "x" & Y & " xc:brown "  & strPs
wsh.run command, 7, true
command = "composite -tile " & strPH & " " & strPs & " "  & strP
wsh.run command, 7, true

' The white strip between the single film strips
Wx = Round(fWx * FrY)
command = strConv & " -size " & Wx & "x"  & Y & " xc:white "  & strW
wsh.run command, 7, true

' Appending the single strips horizontally
command = strConv
for i = 0 to NCols-2
	Command = command & " " & strP & " " & strPre & i & strExt & " " & strP & " " & strW
next
Command = command & " " & strP & " " & strPre & i & strExt & " " & strP & " +append -quality 80% " & strout & "." & strFileExt
Wsh.run command, 7, true

' cleaning up
fs.DeleteFile("*" & strExt)